@@ -12,6 +12,7 @@
 #' \code{\link[stats]{cor}}
 #' @author Vincent Zoonekynd and Tyler Rinker <tyler.rinker@@gmail.com>.
 #' @export
+#' @rdname v_outer
 #' @examples
 #' pooled.sd <- function(x, y) {
 #'     n1 <- length(x)
@@ -55,20 +56,26 @@
 #' v_outer(with(DATA, wfm(state, person)), cos_sim)
 #' with(DATA, Dissimilarity(state, person))
 #' }
-v_outer <- 
+v_outer <-
+function(x, FUN, ...){
+	FUN
+    UseMethod("v_outer")
+}
+
+
+#' @export
+#' @method v_outer list
+#' @rdname v_outer
+v_outer.list <- 
 function(x, FUN, ...){
     FUN <- match.fun(FUN)
-    if (is.matrix(x)) {
-        x <- as.data.frame(x)
-    }
-    if (is.list(x) & !is.data.frame(x)){
-        if (is.null(names(x))) {
-            names(x) <- paste0("X", seq_along(x))
-        }
-        nms <- names(x)   
+
+    if (is.null(names(x))) {
+        nms <- names(x) <- paste0("X", seq_along(x))
     } else {
-        nms <- colnames(x)
+        nms <- names(x)
     }
+
     z <- outer(
       nms, 
       nms, 
@@ -79,6 +86,51 @@ function(x, FUN, ...){
     z
 }
 
+
+#' @export
+#' @method v_outer data.frame 
+#' @rdname v_outer
+v_outer.data.frame <- 
+function(x, FUN, ...){
+    FUN <- match.fun(FUN)
+	
+    nms <- colnames(x)
+
+    z <- outer(
+      nms, 
+      nms, 
+      Vectorize(function(i,j) FUN(x[, i], x[, j], ...))
+    )
+    dimnames(z) <- list(nms, nms)
+    class(z) <- "v_outer"
+    z
+}
+
+
+#' @export
+#' @method v_outer matrix
+#' @rdname v_outer
+v_outer.matrix <- 
+function(x, FUN, ...){
+    FUN <- match.fun(FUN)
+
+    if (is.null(colnames(x))) {
+        nms <- colnames(x) <- paste0("X", 1:ncol(x))
+    } else {
+        nms <- colnames(x)
+    }
+
+    z <- outer(
+      nms, 
+      nms, 
+      Vectorize(function(i,j) FUN(x[, i], x[, j], ...))
+    )
+
+    dimnames(z) <- list(nms, nms)
+    class(z) <- c("v_outer", class(z))
+    z
+}
+
 #' Prints a v_outer Object.
 #' 
 #' Prints a v_outer object.
